%                 Damped Newton Method (DNM)
%
% This code was used to generate the numerical experiments related to fuction
%
%             f_2(X) = c*logm(det(X))-d*trace(X);
%  
% in the paper 
% 
%       Damped Newton Method on Riemannian Manifolds
% Bortoloti, M. A. A, Fernandes, T. A., Ferreira, O. P. Yun, Y. J.
% 
%
clc;
clear all;
eps1 = 10^(-8);% Tolerance gradient norm of f_1.
eps2 = 10^(-6);
MAX_k = 1000;      % Maximum number of iterations. 
MAX_j = 100;       % Maximum number related to linear search.
sigma = 0.001;     % Parameter of linear search.

n = 100;
d = 1; 
lambda = 500;
c = d*lambda;
fprintf('Dim = %5d\n',n);    
arquivo = strcat('matrix',int2str(n),'.dat');   
address = pwd;
input_file = strcat(address,'/initial_guess/',arquivo);
%*************************************************************************
%                           Load intial guess
%
fin = fopen(input_file); 
P = fscanf(fin,'%f',[n,n]);    
fclose(fin); 
m = -n;
for i =1:n
    m = m + n;
    for j = 1:n
      P1(i,j) = P(j+m);
    end
end
%*************************************************************************
I = eye(n); % Identity matrix n by n.
GRADrmf=c*P1-d*P1^(2); % Riemannian gradient of f_2.
clear iter;
k = 1; % Number of iterates.
iP1 = inv(P1); % Inverse matrix of P1.          
iter(k) =  sqrt(trace(GRADrmf*iP1*GRADrmf*iP1)); % Riemannian gradient norm of f_2.
fprintf('%5d %20.15f\n',k-1,iter(k));
FE = 0; % Function evalution in linear search.
j=0;
tic % Start the time evaluation.
while( iter(k) > eps1  && k < MAX_k && j < MAX_j)
    try
    V = sylvester(P1,P1,2*(c/d*P1-P1^(2))); % Newton's equation.
    catch
        fprintf('Sylvester not solved\n')
        signal2 = 1;
        break;
    end
    
    if(norm(V-V','fro') < eps2) % Check if V belongs to tangent space.
      j = armijo_newton_direction_dnm_f2(P1,V,sigma,c,d,MAX_j); % Linear search.
      FE = 3 + j + FE; % Update function evaluation number.
      SQRTP = sqrtm(P1);  
      iSQRTP= inv(SQRTP);
      EXP_M = expm(2^(-j)*iSQRTP*V*iSQRTP);
      P2 = SQRTP*EXP_M*SQRTP;
    else
        V = d^(2)*P1^(3)-c*d*P1^(2); % Gradient direction of merit function.
        j = armijo_grad_direction_dnm_f2(P1,V,sigma,c,d,MAX_j); % Linear search.
        SQRTP = sqrtm(P1);  
        iSQRTP= inv(SQRTP);
        EXP_M = expm(-2^(-j)*iSQRTP*V*iSQRTP);  
        P2 = SQRTP*EXP_M*SQRTP;
    end
    k = k + 1;
    GRADrmf=c*P2-d*P2^(2); % Riemannian gradient at new iteration.
    if(isnan(GRADrmf))
        signal2 = 1;
        break;
    end
    iP2 = inv(P2);
    if(isnan(iP2))
        break;
    end;
    iter(k) = sqrt(trace(GRADrmf*iP2*GRADrmf*iP2)); %Riemannian gradient norm of f_1 at new iteration.
    fprintf('%5d %20.15f\n',k-1,iter(k));
    P1 = P2;
end
t = toc; % Ends time evaluation.
fprintf('CPU time %f\n',t);
fprintf('FE %d\n',FE);
fprintf('The solution found can be seen in P1 variable.\n');