%                   Newton Method (NM)
%
% This code was used to generate the numerical experiments related to fuction
%
%       f_1(x) = a*log(det(X))+b*trace(X^(-1))
%  
% in the paper 
% 
%       Damped Newton Method on Riemannian Manifolds
% Bortoloti, M. A. A, Fernandes, T. A., Ferreira, O. P. Yun, Y. J.
% 
%
clc;
clear all;
eps1 = 10^(-8);% Tolerance gradient norm of f_1.
eps2 = 10^(-6);
MAX_k = 1000;      % Maximum number of iterations. 
MAX_j = 100;       % Maximum number related to linear search.
sigma = 0.001;     % Parameter of linear search.

n = 100;
a = 1; 
lambda = 0.1;
b = a*lambda;
fprintf('Dim = %5d\n',n);    
arquivo = strcat('matrix',int2str(n),'.dat');   
address = pwd;
input_file = strcat(address,'/initial_guess/',arquivo);
%*************************************************************************
%                           Load intial guess
%
fin = fopen(input_file); 
P = fscanf(fin,'%f',[n,n]);    
fclose(fin); 
m = -n;
for i =1:n
    m = m + n;
    for j = 1:n
      P1(i,j) = P(j+m);
    end
end
%*************************************************************************
I = eye(n); % Identity matrix n by n.
GRADrmf=a*P1-b*I; % Riemannian gradient of f_1.
clear iter;
k = 1; % Number of iterates.
iP1 = inv(P1); % Inverse matrix of P1.
iter(k) =  sqrt(trace(GRADrmf*iP1*GRADrmf*iP1)); % Riemannian gradient norm of f_1.
fprintf('%5d %20.15f\n',k-1,iter(k));
tic % Start the time evaluation.
while( iter(k) > eps1  && k < MAX_k)
    try
    V = sylvester(P1,P1,2*(P1^2-a/b*P1^3)); % Newton's equation.
    catch
        fprintf('Sylvester not solved\n')
        signal2 = 1;
        break;
    end
      SQRTP = sqrtm(P1);  
      iSQRTP= inv(SQRTP);
      EXP_M = expm(iSQRTP*V*iSQRTP);  
      P2 = SQRTP*EXP_M*SQRTP;
    k = k + 1;
    GRADrmf=a*P2-b*I; % Riemannian gradient at new iteration.
    if(isnan(GRADrmf))
        signal2 = 1;
        break;
    end
    iP2 = inv(P2);
    if(isnan(iP2))
        break;
    end;
    iter(k) = sqrt(trace(GRADrmf*iP2*GRADrmf*iP2)); % Riemannian gradient norm of f_1 at new iteration.
    fprintf('%5d %20.15f\n',k-1,iter(k));
    P1 = P2;
end
t = toc; % Ends time evaluation.
fprintf('CPU time %f\n',t);
fprintf('The solution found can be seen in P1 variable.\n');